@@ -6,6 +6,7 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmfileutil.h>
 #include <rpm/rpmmacro.h>
+#include <rpm/rpmstring.h>
 #include "lib/rpmdb_internal.h"
 
 #include "debug.h"
@@ -179,9 +180,8 @@ static int sqlite_init(rpmdb rdb, const char * dbhome)
 		int one = 1;
 		/* Annoying but necessary to support non-privileged readers */
 		sqlite3_file_control(sdb, NULL, SQLITE_FCNTL_PERSIST_WAL, &one);
-
-		if (!rpmExpandNumeric("%{?_flush_io}"))
-		    sqlexec(sdb, "PRAGMA wal_autocheckpoint = 0");
+		/* Sqlite default threshold is way too low for rpmdb */
+		sqlexec(sdb, "PRAGMA wal_autocheckpoint = 10000");
 	    }
 	}
 
