@@ -10,6 +10,7 @@
 #include <rpm/rpmdb.h>
 #include <rpm/rpmds.h>
 #include <rpm/rpmfi.h>
+#include <rpm/rpmstring.h>
 
 #include "lib/rpmts_internal.h"
 #include "lib/rpmte_internal.h"
@@ -97,6 +98,21 @@ static rpmRC headerCheckPayloadFormat(Header h) {
     return rc;
 }
 
+static void addElement(tsMembers tsmem, rpmte te, int oc)
+{
+    if (oc >= tsmem->orderAlloced) {
+	tsmem->orderAlloced += (oc - tsmem->orderAlloced) + tsmem->delta;
+	tsmem->order = xrealloc(tsmem->order,
+	tsmem->orderAlloced * sizeof(*tsmem->order));
+    }
+
+    tsmem->order[oc] = te;
+    if (oc == tsmem->orderCount) {
+	tsmem->orderCount++;
+    }
+}
+
+
 /**
  * Add removed package instance to ordered transaction set.
  * @param ts		transaction set
@@ -125,16 +141,9 @@ static int removePackage(rpmts ts, Header h, rpmte depends)
 	return 1;
 
     packageHashAddEntry(tsmem->removedPackages, dboffset, p);
-
-    if (tsmem->orderCount >= tsmem->orderAlloced) {
-	tsmem->orderAlloced += (tsmem->orderCount - tsmem->orderAlloced) + tsmem->delta;
-	tsmem->order = xrealloc(tsmem->order, sizeof(*tsmem->order) * tsmem->orderAlloced);
-    }
-
     rpmteSetDependsOn(p, depends);
 
-    tsmem->order[tsmem->orderCount] = p;
-    tsmem->orderCount++;
+    addElement(tsmem, p, tsmem->orderCount);
     rpmtsNotifyChange(ts, RPMTS_EVENT_ADD, p, depends);
 
     return 0;
@@ -441,17 +450,7 @@ static int addPackage(rpmts ts, Header h,
 	}
     }
 
-    if (oc >= tsmem->orderAlloced) {
-	tsmem->orderAlloced += (oc - tsmem->orderAlloced) + tsmem->delta;
-	tsmem->order = xrealloc(tsmem->order,
-			tsmem->orderAlloced * sizeof(*tsmem->order));
-    }
-
-
-    tsmem->order[oc] = p;
-    if (oc == tsmem->orderCount) {
-	tsmem->orderCount++;
-    }
+    addElement(tsmem, p, oc);
     rpmtsNotifyChange(ts, RPMTS_EVENT_ADD, p, NULL);
 
     
@@ -489,6 +488,22 @@ int rpmtsAddReinstallElement(rpmts ts, Header h, fnpyKey key)
     return addPackage(ts, h, key, RPMTE_REINSTALL, NULL);
 }
 
+int rpmtsAddRestoreElement(rpmts ts, Header h)
+{
+    tsMembers tsmem = rpmtsMembers(ts);
+    if (rpmtsSetupTransactionPlugins(ts) == RPMRC_FAIL)
+	return 1;
+
+    rpmte p = rpmteNew(ts, h, TR_RESTORED, NULL, NULL, 0);
+    if (p == NULL)
+	return 1;
+
+    addElement(tsmem, p, tsmem->orderCount);
+    rpmtsNotifyChange(ts, RPMTS_EVENT_ADD, p, NULL);
+
+    return 0;
+}
+
 int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
 {
     if (rpmtsSetupTransactionPlugins(ts) == RPMRC_FAIL)
