@@ -2,7 +2,7 @@
 #define H_MISC
 
 /**
- * \file lib/misc.h
+ * \file misc.h
  *
  */
 
@@ -25,7 +25,7 @@ RPM_GNUC_INTERNAL
 char * rpmVerifyString(uint32_t verifyResult, const char *pad);
 
 RPM_GNUC_INTERNAL
-char * rpmFFlagsString(uint32_t fflags, const char *pad);
+char * rpmFFlagsString(uint32_t fflags);
 
 typedef int (*headerTagTagFunction) (Header h, rpmtd td, headerGetFlags hgflags);
 
@@ -62,8 +62,6 @@ RPM_GNUC_INTERNAL
 void rpmRelocationBuild(Header h, rpmRelocation *rawrelocs,
 		int *rnrelocs, rpmRelocation **rrelocs, uint8_t **rbadrelocs);
 
-RPM_GNUC_INTERNAL
-void rpmAtExit(void);
 #ifdef __cplusplus
 }
 #endif
