@@ -94,6 +94,10 @@ static inline int addSingleRelation(rpmte p,
 	    RPMSENSE_SCRIPT_PRE : RPMSENSE_SCRIPT_PREUN;
     }
 
+    /* Avoid loop-breaker inflation from weak dependencies for now */
+    if (rpmdsIsWeak(dep))
+	flags = 0;
+
     if (reversed) {
 	rpmte r = p;
 	p = q;
@@ -609,6 +613,14 @@ int rpmtsOrder(rpmts ts)
 
     rpmlog(RPMLOG_DEBUG, "========== tsorting packages (order, #predecessors, #succesors, depth)\n");
 
+    /* Restored items first (doesn't matter but is simple) */
+    for (int e = 0; e < nelem; e++) {
+	tsortInfo p = &sortInfo[e];
+	if (rpmteType(p->te) == TR_RESTORED) {
+	    newOrder[newOrderCount++] = p->te;
+	}
+    }
+
     for (int i = 0; i < 2; i++) {
 	/* Do two separate runs: installs first - then erases */
 	int oType = !i ? TR_ADDED : TR_REMOVED;
