@@ -24,8 +24,6 @@
 
 #include "debug.h"
 
-int _print_pkts = 0;
-
 static int doImport(rpmts ts, const char *fn, char *buf, ssize_t blen)
 {
     char const * const pgpmark = "-----BEGIN PGP ";
@@ -92,19 +90,6 @@ int rpmcliImportPubkeys(rpmts ts, ARGV_const_t argv)
 	char *t = NULL;
 	int iorc;
 
-	/* If arg looks like a keyid, then attempt keyserver retrieve. */
-	if (rstreqn(fn, "0x", 2)) {
-	    const char * s = fn + 2;
-	    int i;
-	    for (i = 0; *s && isxdigit(*s); s++, i++)
-		{};
-	    if (i == 8 || i == 16) {
-		t = rpmExpand("%{_hkp_keyserver_query}", fn+2, NULL);
-		if (t && *t != '%')
-		    fn = t;
-	    }
-	}
-
 	/* Read the file and try to import all contained keys */
 	iorc = rpmioSlurp(fn, &buf, &blen);
 	if (iorc || buf == NULL || blen < 64) {
@@ -187,11 +172,12 @@ rpmRC rpmpkgRead(struct rpmvs_s *vs, FD_t fd,
     /* Finalize header range */
     rpmvsFiniRange(vs, RPMSIG_HEADER);
 
-    /* Unless disabled, read the payload, generating digest(s) on the fly. */
-    if (!(rpmvsFlags(vs) & RPMVSF_NEEDPAYLOAD)) {
-	/* Fish interesting tags from the main header. This is a bit hacky... */
-	rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGEST);
+    /* Fish interesting tags from the main header. This is a bit hacky... */
+    rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGEST);
+    rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGESTALT);
 
+    /* If needed and not explicitly disabled, read the payload as well. */
+    if (rpmvsRange(vs) & RPMSIG_PAYLOAD) {
 	/* Initialize digests ranging over the payload only */
 	rpmvsInitRange(vs, RPMSIG_PAYLOAD);
 
@@ -267,8 +253,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd, const char * fn)
     int rc = 1; /* assume failure */
     if (ts && qva && fd && fn) {
 	rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
+	rpmVSFlags vsflags = rpmtsVfyFlags(ts);
 	int vfylevel = rpmtsVfyLevel(ts);
-	rc = rpmpkgVerifySigs(keyring, vfylevel, qva->qva_flags, fd, fn);
+	rc = rpmpkgVerifySigs(keyring, vfylevel, vsflags, fd, fn);
     	rpmKeyringFree(keyring);
     }
     return rc;
@@ -282,10 +269,6 @@ int rpmcliVerifySignatures(rpmts ts, ARGV_const_t argv)
     rpmVSFlags vsflags = rpmtsVfyFlags(ts);
     int vfylevel = rpmtsVfyLevel(ts);
 
-    if (rpmcliQueryFlags & QUERY_DIGEST)
-	vsflags |= _RPMVSF_NODIGESTS;
-    if (rpmcliQueryFlags & QUERY_SIGNATURE)
-	vsflags |= _RPMVSF_NOSIGNATURES;
     vsflags |= rpmcliVSFlags;
     if (rpmcliVfyLevelMask) {
 	vfylevel &= ~rpmcliVfyLevelMask;
@@ -303,7 +286,6 @@ int rpmcliVerifySignatures(rpmts ts, ARGV_const_t argv)
 	}
 
 	Fclose(fd);
-	rpmsqPoll();
     }
     rpmKeyringFree(keyring);
     return res;
