@@ -18,8 +18,7 @@
 
 #include <rpm/rpmtypes.h>
 #include <rpm/rpmurl.h>
-#include <rpm/rpmpgp.h>
-#include <rpm/rpmpgp.h>
+#include <rpm/rpmcrypto.h>
 #include <rpm/rpmmacro.h>
 #include <rpm/rpmsq.h>
 #include <rpm/rpmstring.h>
@@ -303,26 +302,6 @@ struct rpmdbIndexIterator_s {
     int			ii_skipdata;
 };
 
-static rpmdb rpmdbRock;
-static rpmdbMatchIterator rpmmiRock;
-static rpmdbIndexIterator rpmiiRock;
-
-void rpmAtExit(void)
-{
-    rpmdb db;
-    rpmdbMatchIterator mi;
-    rpmdbIndexIterator ii;
-
-    while ((mi = rpmmiRock) != NULL)
-	rpmdbFreeIterator(mi);
-
-    while ((ii = rpmiiRock) != NULL)
-	rpmdbIndexIteratorFree(ii);
-
-    while ((db = rpmdbRock) != NULL)
-	(void) rpmdbClose(db);
-}
-
 rpmop rpmdbOp(rpmdb rpmdb, rpmdbOpX opx)
 {
     rpmop op = NULL;
@@ -386,18 +365,11 @@ static int dbiForeach(dbiIndex *dbis, int ndbi,
 
 int rpmdbClose(rpmdb db)
 {
-    rpmdb * prev, next;
     int rc = 0;
 
     if (db == NULL)
 	goto exit;
 
-    prev = &rpmdbRock;
-    while ((next = *prev) != NULL && next != db)
-	prev = &next->db_next;
-    if (!next)
-	goto exit;
-
     (void) rpmdbUnlink(db);
 
     if (db->nrefs > 0)
@@ -417,16 +389,8 @@ int rpmdbClose(rpmdb db)
     db->db_checked = dbChkFree(db->db_checked);
     db->db_indexes = _free(db->db_indexes);
 
-    if (next) {
-        *prev = next->db_next;
-	next->db_next = NULL;
-    }
-
     db = _free(db);
 
-    if (rpmdbRock == NULL) {
-	rpmsqActivate(0);
-    }
 exit:
     return rc;
 }
@@ -499,20 +463,12 @@ static int openDatabase(const char * prefix,
     if (db == NULL)
 	return 1;
 
-    db->db_next = rpmdbRock;
-    rpmdbRock = db;
-
     /* Try to ensure db home exists, error out if we can't even create */
     rc = rpmioMkpath(rpmdbHome(db), 0755, getuid(), getgid());
     if (rc == 0) {
 	/* Open just bare minimum when rebuilding a potentially damaged db */
 	int justPkgs = (db->db_flags & RPMDB_FLAG_REBUILD) &&
 		       ((db->db_mode & O_ACCMODE) == O_RDONLY);
-	/* Enable signal queue on the first db open */
-	if (db->db_next == NULL) {
-	    rpmsqActivate(1);
-	}
-
 	rc = doOpen(db, justPkgs);
 
 	if (!db->db_descr)
@@ -997,22 +953,12 @@ static int miFreeHeader(rpmdbMatchIterator mi, dbiIndex dbi)
 
 rpmdbMatchIterator rpmdbFreeIterator(rpmdbMatchIterator mi)
 {
-    rpmdbMatchIterator * prev, next;
     dbiIndex dbi = NULL;
     int i;
 
     if (mi == NULL)
 	return NULL;
 
-    prev = &rpmmiRock;
-    while ((next = *prev) != NULL && next != mi)
-	prev = &next->mi_next;
-    if (next) {
-	*prev = next->mi_next;
-	next->mi_next = NULL;
-    } else
-	return NULL;
-
     pkgdbOpen(mi->mi_db, 0, &dbi);
 
     miFreeHeader(mi, dbi);
@@ -1036,8 +982,6 @@ rpmdbMatchIterator rpmdbFreeIterator(rpmdbMatchIterator mi)
 
     mi = _free(mi);
 
-    (void) rpmsqPoll();
-
     return NULL;
 }
 
@@ -1163,6 +1107,7 @@ static char * mireDup(rpmTagVal tag, rpmMireMode *modep,
 	    case '.':
 	    case '+':
 	    case '*':
+	    case '^':
 		if (!brackets) nb++;
 		break;
 	    case '\\':
@@ -1189,6 +1134,7 @@ static char * mireDup(rpmTagVal tag, rpmMireMode *modep,
 	    switch (*s) {
 	    case '.':
 	    case '+':
+	    case '^':
 		if (!brackets) *t++ = '\\';
 		break;
 	    case '*':
@@ -1682,10 +1628,6 @@ rpmdbMatchIterator rpmdbNewIterator(rpmdb db, rpmDbiTagVal dbitag)
     mi->mi_ts = NULL;
     mi->mi_hdrchk = NULL;
 
-    /* Chain cursors for teardown on abnormal exit. */
-    mi->mi_next = rpmmiRock;
-    rpmmiRock = mi;
-
     return mi;
 };
 
@@ -1762,8 +1704,6 @@ rpmdbMatchIterator rpmdbInitIterator(rpmdb db, rpmDbiTagVal rpmtag,
     rpmdbMatchIterator mi = NULL;
 
     if (db != NULL) {
-	(void) rpmsqPoll();
-
 	if (rpmtag == RPMDBI_PACKAGES)
 	    mi = pkgdbIterInit(db, keyp, keylen);
 	else
@@ -1785,8 +1725,6 @@ rpmdbMatchIterator rpmdbInitPrefixIterator(rpmdb db, rpmDbiTagVal rpmtag,
 	return NULL;
 
     if (db != NULL && rpmtag != RPMDBI_PACKAGES) {
-	(void) rpmsqPoll();
-
 
 	if (indexOpen(db, dbtag, 0, &dbi) == 0) {
 	    int rc = 0;
@@ -1871,16 +1809,10 @@ rpmdbIndexIterator rpmdbIndexIteratorInit(rpmdb db, rpmDbiTag rpmtag)
     if (db == NULL)
 	return NULL;
 
-    (void) rpmsqPoll();
-
     if (indexOpen(db, rpmtag, 0, &dbi))
 	return NULL;
 
-    /* Chain cursors for teardown on abnormal exit. */
     ii = xcalloc(1, sizeof(*ii));
-    ii->ii_next = rpmiiRock;
-    rpmiiRock = ii;
-
     ii->ii_db = rpmdbLink(db);
     ii->ii_rpmtag = rpmtag;
     ii->ii_dbi = dbi;
@@ -2008,20 +1940,9 @@ unsigned int rpmdbIndexIteratorTagNum(rpmdbIndexIterator ii, unsigned int nr)
 
 rpmdbIndexIterator rpmdbIndexIteratorFree(rpmdbIndexIterator ii)
 {
-    rpmdbIndexIterator * prev, next;
-
     if (ii == NULL)
         return NULL;
 
-    prev = &rpmiiRock;
-    while ((next = *prev) != NULL && next != ii)
-        prev = &next->ii_next;
-    if (next) {
-        *prev = next->ii_next;
-        next->ii_next = NULL;
-    } else
-	return NULL;
-
     ii->ii_dbc = dbiCursorFree(ii->ii_dbi, ii->ii_dbc);
     ii->ii_dbi = NULL;
     rpmdbClose(ii->ii_db);
@@ -2643,7 +2564,7 @@ char *rpmdbCookie(rpmdb db)
     rpmdbIndexIterator ii = rpmdbIndexIteratorInit(db, RPMDBI_NAME);
 
     if (ii) {
-	DIGEST_CTX ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
+	DIGEST_CTX ctx = rpmDigestInit(RPM_HASH_SHA256, RPMDIGEST_NONE);
 	const void *key = 0;
 	size_t keylen = 0;
 	while ((rpmdbIndexIteratorNext(ii, &key, &keylen)) == 0) {
