@@ -4,12 +4,15 @@
 
 #include "system.h"
 
+#include <string.h>
+
 #include <rpm/rpmcli.h>
 #include <rpm/rpmtag.h>
 #include <rpm/rpmlib.h>		/* rpmReadPackageFile, vercmp etc */
 #include <rpm/rpmdb.h>
 #include <rpm/rpmds.h>
 #include <rpm/rpmts.h>
+#include <rpm/rpmsq.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmfileutil.h>
 
@@ -82,12 +85,7 @@ static rpmVSFlags setvsFlags(struct rpmInstallArguments_s * ia)
     else
 	vsflags = rpmExpandNumeric("%{?_vsflags_install}");
 
-    if (rpmcliQueryFlags & VERIFY_DIGEST)
-	vsflags |= _RPMVSF_NODIGESTS;
-    if (rpmcliQueryFlags & VERIFY_SIGNATURE)
-	vsflags |= _RPMVSF_NOSIGNATURES;
-    if (rpmcliQueryFlags & VERIFY_HDRCHK)
-	vsflags |= RPMVSF_NOHDRCHK;
+    vsflags |= rpmcliVSFlags;
 
     return vsflags;
 }
@@ -171,6 +169,7 @@ void * rpmShowProgress(const void * arg,
     case RPMCALLBACK_TRANS_PROGRESS:
     case RPMCALLBACK_INST_PROGRESS:
     case RPMCALLBACK_UNINST_PROGRESS:
+    case RPMCALLBACK_VERIFY_PROGRESS:
 	if (flags & INSTALL_PERCENT)
 	    fprintf(stdout, "%%%% %f\n", (double) (total
 				? ((((float) amount) / total) * 100)
@@ -181,6 +180,7 @@ void * rpmShowProgress(const void * arg,
 	break;
 
     case RPMCALLBACK_TRANS_START:
+    case RPMCALLBACK_VERIFY_START:
 	rpmcliHashesCurrent = 0;
 	rpmcliProgressTotal = 1;
 	rpmcliProgressCurrent = 0;
@@ -188,14 +188,18 @@ void * rpmShowProgress(const void * arg,
 	rpmcliProgressState = what;
 	if (!(flags & INSTALL_LABEL))
 	    break;
-	if (flags & INSTALL_HASH)
-	    fprintf(stdout, "%-38s", _("Preparing..."));
-	else
-	    fprintf(stdout, "%s\n", _("Preparing packages..."));
+	if (flags & INSTALL_HASH) {
+	    fprintf(stdout, "%-38s", (what == RPMCALLBACK_TRANS_START) ?
+		    _("Preparing...") : _("Verifying..."));
+	} else {
+	    fprintf(stdout, "%s\n", (what == RPMCALLBACK_TRANS_START) ?
+		    _("Preparing packages...") : _("Verifying packages..."));
+	}
 	(void) fflush(stdout);
 	break;
 
     case RPMCALLBACK_TRANS_STOP:
+    case RPMCALLBACK_VERIFY_STOP:
 	if (flags & INSTALL_HASH)
 	    printHash(1, 1);	/* Fixes "preparing..." progress bar */
 	rpmcliProgressTotal = rpmcliPackagesTotal;
@@ -248,10 +252,10 @@ struct rpmEIU {
     rpmRC rpmrc;
 };
 
-static int rpmcliTransaction(rpmts ts, struct rpmInstallArguments_s * ia,
-		      int numPackages)
+static int rpmcliTransaction(rpmts ts, struct rpmInstallArguments_s * ia)
 {
     rpmps ps;
+    int numPackages = rpmtsNElements(ts);
 
     int rc = 0;
     int stop = 0;
@@ -283,15 +287,22 @@ static int rpmcliTransaction(rpmts ts, struct rpmInstallArguments_s * ia,
     }
 
     if (numPackages && !stop) {
-	rpmlog(RPMLOG_DEBUG, eflags ? "erasing packages\n" :
-				      "installing binary packages\n");
+	const char *msg;
+	if (ia->installInterfaceFlags & INSTALL_RESTORE)
+	    msg = "restoring packages";
+	else if (ia->installInterfaceFlags & INSTALL_ERASE)
+	    msg = "erasing packages";
+	else
+	    msg = "installing binary packages";
+
+	rpmlog(RPMLOG_DEBUG, "%s\n", msg);
 	rpmtsClean(ts);
 	rc = rpmtsRun(ts, NULL, ia->probFilter);
 
 	ps = rpmtsProblems(ts);
 
-	if ((rpmpsNumProblems(ps) > 0) && (eflags? 1 : (rc > 0)))
-	    rpmpsPrint((eflags? NULL : stderr), ps);
+	if (rpmpsNumProblems(ps) > 0 && (eflags || rc > 0))
+	    rpmpsPrint(NULL, ps);
 	ps = rpmpsFree(ps);
     }
 
@@ -311,7 +322,8 @@ static int tryReadManifest(struct rpmEIU * eiu)
 	    Fclose(fd);
 	    fd = NULL;
 	}
-	eiu->numFailed++; *eiu->fnp = NULL;
+	eiu->numFailed++;
+	*eiu->fnp = NULL;
 	return RPMRC_FAIL;
     }
 
@@ -323,8 +335,10 @@ static int tryReadManifest(struct rpmEIU * eiu)
     Fclose(fd);
     fd = NULL;
 
-    if (rc != RPMRC_OK)
-        eiu->numFailed++; *eiu->fnp = NULL;
+    if (rc != RPMRC_OK) {
+        eiu->numFailed++;
+        *eiu->fnp = NULL;
+    }
 
     return rc;
 }
@@ -340,7 +354,8 @@ static int tryReadHeader(rpmts ts, struct rpmEIU * eiu, Header * hdrp)
            Fclose(fd);
 	   fd = NULL;
        }
-       eiu->numFailed++; *eiu->fnp = NULL;
+       eiu->numFailed++;
+       *eiu->fnp = NULL;
        return RPMRC_FAIL;
    }
 
@@ -353,9 +368,10 @@ static int tryReadHeader(rpmts ts, struct rpmEIU * eiu, Header * hdrp)
    if (eiu->rpmrc == RPMRC_NOTFOUND && (giFlags & RPMGI_NOMANIFEST))
        eiu->rpmrc = RPMRC_FAIL;
 
-   if(eiu->rpmrc == RPMRC_FAIL) {
+   if (eiu->rpmrc == RPMRC_FAIL) {
        rpmlog(RPMLOG_ERR, _("%s cannot be installed\n"), *eiu->fnp);
-       eiu->numFailed++; *eiu->fnp = NULL;
+       eiu->numFailed++;
+       *eiu->fnp = NULL;
    }
 
    return RPMRC_OK;
@@ -363,19 +379,21 @@ static int tryReadHeader(rpmts ts, struct rpmEIU * eiu, Header * hdrp)
 
 
 /* On --freshen, verify package is installed and newer */
-static int checkFreshenStatus(rpmts ts, Header h)
+static int checkFreshenStatus(rpmts ts, Header h, int allowOld)
 {
     rpmdbMatchIterator mi = NULL;
     const char * name = headerGetString(h, RPMTAG_NAME);
     const char *arch = headerGetString(h, RPMTAG_ARCH);
     Header oldH = NULL;
-
     if (name != NULL)
         mi = rpmtsInitIterator(ts, RPMDBI_NAME, name, 0);
     if (rpmtsColor(ts) && arch)
 	rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
 
     while ((oldH = rpmdbNextIterator(mi)) != NULL) {
+	/* Package is older than those currently installed. */
+	if (allowOld && rpmVersionCompare(oldH, h) > 0)
+	    break;
 	/* Package is newer than those currently installed. */
         if (rpmVersionCompare(oldH, h) < 0)
 	    break;
@@ -385,6 +403,19 @@ static int checkFreshenStatus(rpmts ts, Header h)
     return (oldH != NULL);
 }
 
+static int rpmNoGlob(const char *fn, int *argcPtr, ARGV_t * argvPtr)
+{
+    struct stat sb;
+    int rc = stat(fn, &sb);
+    if (rc == 0) {
+	argvAdd(argvPtr, fn);
+	*argcPtr = 1;
+    } else {
+	*argcPtr = 0;
+    }
+    return rc;
+}
+
 /** @todo Generalize --freshen policies. */
 int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
 {
@@ -392,14 +423,23 @@ int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
     rpmRelocation * relocations;
     char * fileURL = NULL;
     rpmVSFlags vsflags, ovsflags;
+    rpmVSFlags ovfyflags;
     int rc;
     int i;
 
     vsflags = setvsFlags(ia);
     ovsflags = rpmtsSetVSFlags(ts, (vsflags | RPMVSF_NEEDPAYLOAD));
+    /* for rpm cli, --nosignature/--nodigest applies to both vs and vfyflags */
+    ovfyflags = rpmtsSetVfyFlags(ts, (rpmtsVfyFlags(ts) | rpmcliVSFlags));
 
     if (fileArgv == NULL) goto exit;
 
+    if (rpmcliVfyLevelMask) {
+	int vfylevel = rpmtsVfyLevel(ts);
+	vfylevel &= ~rpmcliVfyLevelMask;
+	rpmtsSetVfyLevel(ts, vfylevel);
+    }
+
     (void) rpmtsSetFlags(ts, ia->transFlags);
 
     relocations = ia->relocations;
@@ -417,14 +457,22 @@ int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
     for (eiu->fnp = fileArgv; *eiu->fnp != NULL; eiu->fnp++) {
     	ARGV_t av = NULL;
     	int ac = 0;
-	char * fn;
 
-	fn = rpmEscapeSpaces(*eiu->fnp);
-	rc = rpmGlob(fn, &ac, &av);
-	fn = _free(fn);
+	if (giFlags & RPMGI_NOGLOB) {
+	    rc = rpmNoGlob(*eiu->fnp, &ac, &av);
+	} else {
+	    char * fn = rpmEscapeSpaces(*eiu->fnp);
+	    rc = rpmGlob(fn, &ac, &av);
+	    fn = _free(fn);
+	}
 	if (rc || ac == 0) {
-	    rpmlog(RPMLOG_ERR, _("File not found by glob: %s\n"), *eiu->fnp);
+	    if (giFlags & RPMGI_NOGLOB) {
+		rpmlog(RPMLOG_ERR, _("File not found: %s\n"), *eiu->fnp);
+	    } else {
+		rpmlog(RPMLOG_ERR, _("File not found by glob: %s\n"), *eiu->fnp);
+	    }
 	    eiu->numFailed++;
+	    argvFree(av);
 	    continue;
 	}
 
@@ -520,6 +568,10 @@ restart:
 	}
 
 	if (headerIsSource(h)) {
+	    if (ia->installInterfaceFlags & INSTALL_FRESHEN) {
+		headerFree(h);
+	        continue;
+	    }
 	    rpmlog(RPMLOG_DEBUG, "\tadded source package [%d]\n",
 		eiu->numSRPMS);
 	    eiu->sourceURL = xrealloc(eiu->sourceURL,
@@ -547,7 +599,7 @@ restart:
 	}
 
 	if (ia->installInterfaceFlags & INSTALL_FRESHEN)
-	    if (checkFreshenStatus(ts, h) != 1) {
+	    if (checkFreshenStatus(ts, h, ia->probFilter & RPMPROB_FILTER_OLDPACKAGE) != 1) {
 		headerFree(h);
 	        continue;
 	    }
@@ -563,7 +615,7 @@ restart:
 	if (eiu->relocations)
 	    eiu->relocations->oldPath = _free(eiu->relocations->oldPath);
 
-	switch(rc) {
+	switch (rc) {
 	case 0:
 	    rpmlog(RPMLOG_DEBUG, "\tadded binary package [%d]\n",
 			eiu->numRPMS);
@@ -589,7 +641,7 @@ restart:
     if (eiu->numFailed) goto exit;
 
     if (eiu->numRPMS) {
-        int rc = rpmcliTransaction(ts, ia, eiu->numPkgs);
+        int rc = rpmcliTransaction(ts, ia);
         if (rc < 0)
             eiu->numFailed += eiu->numRPMS;
 	else if (rc > 0)
@@ -601,7 +653,6 @@ restart:
 	rpmcliProgressTotal = 0;
 	rpmcliProgressCurrent = 0;
 	for (i = 0; i < eiu->numSRPMS; i++) {
-	    rpmdbCheckSignals();
 	    if (eiu->sourceURL[i] != NULL) {
 	        rc = RPMRC_OK;
 		if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST))
@@ -629,6 +680,7 @@ exit:
 
     rpmtsEmpty(ts);
     rpmtsSetVSFlags(ts, ovsflags);
+    rpmtsSetVfyFlags(ts, ovfyflags);
 
     return rc;
 }
@@ -686,12 +738,41 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_const_t argv)
     free(qfmt);
 
     if (numFailed) goto exit;
-    numFailed = rpmcliTransaction(ts, ia, numPackages);
+    numFailed = rpmcliTransaction(ts, ia);
 exit:
     rpmtsEmpty(ts);
     rpmtsSetVSFlags(ts, ovsflags);
 
-    return numFailed;
+    return (numFailed < 0) ? numPackages : numFailed;
+}
+
+static int handleRestorePackage(QVA_t qva, rpmts ts, Header h)
+{
+    return rpmtsAddRestoreElement(ts, h);
+}
+
+int rpmRestore(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_const_t argv)
+{
+    QVA_t qva = &rpmQVKArgs;
+    int rc;
+    rpmVSFlags vsflags = setvsFlags(ia);
+    rpmVSFlags ovsflags = rpmtsSetVSFlags(ts, vsflags);
+
+    rpmtsSetFlags(ts, ia->transFlags);
+    setNotifyFlag(ia, ts);
+
+    if (qva->qva_showPackage == NULL)
+	qva->qva_showPackage = handleRestorePackage;
+
+    rc = rpmcliArgIter(ts, qva, argv);
+    if (rc == 0) {
+	rc = rpmcliTransaction(ts, ia);
+    }
+
+    rpmtsEmpty(ts);
+    rpmtsSetVSFlags(ts, ovsflags);
+
+    return rc;
 }
 
 int rpmInstallSource(rpmts ts, const char * arg,
