@@ -4,6 +4,7 @@
 #include <sys/wait.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <lua.h>
 
 #include <rpm/rpmfileutil.h>
 #include <rpm/rpmmacro.h>
@@ -34,7 +35,7 @@ struct rpmScript_s {
     char *body;			/* script body */
     char *descr;		/* description for logging */
     rpmscriptFlags flags;	/* flags to control operation */
-    struct scriptNextFileFunc_s nextFileFunc;  /* input function */
+    struct scriptNextFileFunc_s *nextFileFunc;  /* input function */
 };
 
 struct scriptInfo_s {
@@ -93,6 +94,14 @@ static const struct scriptInfo_s * findTag(rpmTagVal tag)
 	si++;
     return si;
 }
+
+static int next_file(lua_State *L)
+{
+    scriptNextFileFunc nff = lua_touserdata(L, lua_upvalueindex(1));
+    lua_pushstring(L, nff->func(nff->param));
+    return 1;
+}
+
 /**
  * Run internal Lua script.
  */
@@ -101,19 +110,45 @@ static rpmRC runLuaScript(rpmPlugins plugins, ARGV_const_t prefixes,
 		   ARGV_t * argvp, const char *script, int arg1, int arg2,
 		   scriptNextFileFunc nextFileFunc)
 {
+    char *scriptbuf = NULL;
     rpmRC rc = RPMRC_FAIL;
-    rpmlua lua = NULL; /* Global state. */
+    rpmlua lua = rpmluaGetGlobalState();
+    lua_State *L = rpmluaGetLua(lua);
     int cwd = -1;
 
     rpmlog(RPMLOG_DEBUG, "%s: running <lua> scriptlet.\n", sname);
 
-    rpmluaSetNextFileFunc(nextFileFunc->func, nextFileFunc->param);
+    if (nextFileFunc) {
+	lua_getglobal(L, "rpm");
+	lua_pushlightuserdata(L, nextFileFunc);
+	lua_pushcclosure(L, &next_file, 1);
+	lua_setfield(L, -2, "next_file");
+    }
+
+    if (prefixes) {
+	lua_newtable(L);
+	for (ARGV_const_t p = prefixes; p && *p; p++) {
+	    int i = p - prefixes + 1;
+	    lua_pushstring(L, *p);
+	    lua_rawseti(L, -2, i);
+	}
+	lua_setglobal(L, "RPM_INSTALL_PREFIX");
+    }
 
     if (arg1 >= 0)
 	argvAddNum(argvp, arg1);
     if (arg2 >= 0)
 	argvAddNum(argvp, arg2);
 
+    if (arg1 >= 0 || arg2 >= 0) {
+	/* Hack to convert arguments to numbers for backwards compat. Ugh. */
+	rstrscat(&scriptbuf,
+		arg1 >= 0 ? "arg[2] = tonumber(arg[2]);" : "",
+		arg2 >= 0 ? "arg[3] = tonumber(arg[3]);" : "",
+		script, NULL);
+        script = scriptbuf;
+    }
+
     /* Lua scripts can change our cwd and umask, save and restore */
     cwd = open(".", O_RDONLY);
     if (cwd != -1) {
@@ -138,6 +173,18 @@ static rpmRC runLuaScript(rpmPlugins plugins, ARGV_const_t prefixes,
 	close(cwd);
 	umask(oldmask);
     }
+    free(scriptbuf);
+
+    if (prefixes) {
+	lua_pushnil(L);
+	lua_setglobal(L, "RPM_INSTALL_PREFIX");
+    }
+
+    if (nextFileFunc) {
+	lua_pushnil(L);
+	lua_setfield(L, -2, "next_file");
+	lua_pop(L, 1); /* "rpm" global */
+    }
 
     return rc;
 }
@@ -148,14 +195,19 @@ static void doScriptExec(ARGV_const_t argv, ARGV_const_t prefixes,
 			FD_t scriptFd, FD_t out)
 {
     int xx;
+    struct sigaction act;
     sigset_t set;
 
-    /* Unmask all signals, the scripts may need them */
+    /* Unmask most signals, the scripts may need them */
     sigfillset(&set);
+    sigdelset(&set, SIGINT);
+    sigdelset(&set, SIGQUIT);
     sigprocmask(SIG_UNBLOCK, &set, NULL);
 
     /* SIGPIPE is ignored in rpm, reset to default for the scriptlet */
-    (void) signal(SIGPIPE, SIG_DFL);
+    memset(&act, 0, sizeof(act));
+    act.sa_handler = SIG_DFL;
+    sigaction(SIGPIPE, &act, NULL);
 
     rpmSetCloseOnExec();
 
@@ -198,9 +250,6 @@ static void doScriptExec(ARGV_const_t argv, ARGV_const_t prefixes,
     }
 	
     if (chdir("/") == 0) {
-	/* XXX Don't mtrace into children. */
-	unsetenv("MALLOC_CHECK_");
-
 	xx = execv(argv[0], argv);
 	if (xx) {
 	    rpmlog(RPMLOG_ERR,
@@ -321,7 +370,7 @@ static rpmRC runExtScript(rpmPlugins plugins, ARGV_const_t prefixes,
     close(inpipe[0]);
     inpipe[0] = 0;
 
-    if (nextFileFunc->func) {
+    if (nextFileFunc) {
 	while ((line = nextFileFunc->func(nextFileFunc->param)) != NULL) {
 	    size_t size = strlen(line);
 	    size_t ret_size;
@@ -413,9 +462,9 @@ rpmRC rpmScriptRun(rpmScript script, int arg1, int arg2, FD_t scriptFd,
 
     if (rc != RPMRC_FAIL) {
 	if (script_type & RPMSCRIPTLET_EXEC) {
-	    rc = runExtScript(plugins, prefixes, script->descr, lvl, scriptFd, &args, script->body, arg1, arg2, &script->nextFileFunc);
+	    rc = runExtScript(plugins, prefixes, script->descr, lvl, scriptFd, &args, script->body, arg1, arg2, script->nextFileFunc);
 	} else {
-	    rc = runLuaScript(plugins, prefixes, script->descr, lvl, scriptFd, &args, script->body, arg1, arg2, &script->nextFileFunc);
+	    rc = runLuaScript(plugins, prefixes, script->descr, lvl, scriptFd, &args, script->body, arg1, arg2, script->nextFileFunc);
 	}
     }
 
@@ -476,9 +525,6 @@ static rpmScript rpmScriptNew(Header h, rpmTagVal tag, const char *body,
 	script->body = b;
     }
 
-    script->nextFileFunc.func = NULL;
-    script->nextFileFunc.param = NULL;
-
     free(nevra);
     return script;
 }
@@ -486,8 +532,9 @@ static rpmScript rpmScriptNew(Header h, rpmTagVal tag, const char *body,
 void rpmScriptSetNextFileFunc(rpmScript script, char *(*func)(void *),
 			    void *param)
 {
-    script->nextFileFunc.func = func;
-    script->nextFileFunc.param = param;
+    script->nextFileFunc = xmalloc(sizeof(*script->nextFileFunc));
+    script->nextFileFunc->func = func;
+    script->nextFileFunc->param = param;
 }
 
 rpmTagVal triggerDsTag(rpmscriptTriggerModes tm)
@@ -623,6 +670,7 @@ rpmScript rpmScriptFree(rpmScript script)
 	free(script->args);
 	free(script->body);
 	free(script->descr);
+	free(script->nextFileFunc);
 	free(script);
     }
     return NULL;
