@@ -8,13 +8,16 @@
 #include "lib/fprint.h"
 #include "lib/rpmlock.h"
 #include "lib/rpmdb_internal.h"
+#include "lib/rpmscript.h"
+#include "lib/rpmtriggers.h"
 
 typedef struct diskspaceInfo_s * rpmDiskSpaceInfo;
 
 /* Transaction set elements information */
 typedef struct tsMembers_s {
     rpmstrPool pool;		/*!< Global string pool */
-    removedHash removedPackages;	/*!< Set of packages being removed. */
+    packageHash removedPackages;	/*!< Set of packages being removed. */
+    packageHash installedPackages;	/*!< Set of installed packages */
     rpmal addedPackages;	/*!< Set of packages being installed. */
 
     rpmds rpmlib;		/*!< rpmlib() dependency set. */
@@ -24,6 +27,17 @@ typedef struct tsMembers_s {
     int delta;			/*!< Delta for reallocation. */
 } * tsMembers;
 
+typedef struct tsTrigger_s {
+    unsigned int hdrNum;
+    int index;
+} tsTrigger;
+
+typedef struct tsTriggers_s {
+    tsTrigger *trigger;
+    int count;
+    int alloced;
+} tsTriggers;
+
 /** \ingroup rpmts
  * The set of packages to be installed/removed atomically.
  */
@@ -36,6 +50,10 @@ struct rpmts_s {
 
     rpmCallbackFunction notify;	/*!< Callback function. */
     rpmCallbackData notifyData;	/*!< Callback private data. */
+    int notifyStyle;		/*!< Callback style (header vs rpmte) */
+
+    rpmtsChangeFunction change;	/*!< Change callback function. */
+    void *changeData;		/*!< Change callback private data. */
 
     rpmprobFilterFlags ignoreSet;
 				/*!< Bits to filter current problems. */
@@ -57,8 +75,10 @@ struct rpmts_s {
     rpm_color_t prefcolor;	/*!< Preferred file color. */
 
     rpmVSFlags vsflags;		/*!< Signature/digest verification flags. */
-
+    rpmVSFlags vfyflags;	/*!< Package verification flags */
+    int vfylevel;		/*!< Package verification level */
     rpmKeyring keyring;		/*!< Keyring in use. */
+    int keyringtype;		/*!< Keyring type */
 
     ARGV_t netsharedPaths;	/*!< From %{_netsharedpath} */
     ARGV_t installLangs;	/*!< From %{_install_langs} */
@@ -68,6 +88,12 @@ struct rpmts_s {
     rpmPlugins plugins;		/*!< Transaction plugins */
 
     int nrefs;			/*!< Reference count. */
+
+    rpmtriggers trigs2run;   /*!< Transaction file triggers */
+
+    int min_writes;             /*!< macro minimize_writes used */
+
+    time_t overrideTime;	/*!< Time value used when overriding system clock. */
 };
 
 #ifdef __cplusplus
@@ -99,6 +125,18 @@ int rpmtsSolve(rpmts ts, rpmds key);
 
 RPM_GNUC_INTERNAL
 rpmRC rpmtsSetupTransactionPlugins(rpmts ts);
+
+RPM_GNUC_INTERNAL
+rpmRC runScript(rpmts ts, rpmte te, Header h, ARGV_const_t prefixes,
+		       rpmScript script, int arg1, int arg2);
+
+
+RPM_GNUC_INTERNAL
+int rpmtsNotifyChange(rpmts ts, int event, rpmte te, rpmte other);
+
+RPM_GNUC_INTERNAL
+rpm_time_t rpmtsGetTime(rpmts ts, time_t step);
+
 #ifdef __cplusplus
 }
 #endif
