@@ -434,6 +434,9 @@ static void handleInstInstalledFile(const rpmts ts, rpmte p, rpmfiles fi, int fx
     if (XFA_SKIPPING(rpmfsGetAction(fs, fx)))
 	return;
 
+    if (rpmfilesFState(otherFi, ofx) == RPMFILE_STATE_NOTINSTALLED)
+	return;
+
     if (rpmfilesCompare(otherFi, ofx, fi, fx)) {
 	int rConflicts = 1;
 	char rState = RPMFILE_STATE_REPLACED;
@@ -703,6 +706,15 @@ assert(otherFi != NULL);
 	    /* Otherwise, we can just erase. */
 	    rpmfsSetAction(fs, i, FA_ERASE);
 	    break;
+	case TR_RESTORED:
+	    if (XFA_SKIPPING(rpmfsGetAction(fs, i)))
+		break;
+	    if (rpmfilesFState(fi, i) != RPMFILE_STATE_NORMAL) {
+		rpmfsSetAction(fs, i, FA_SKIP);
+		break;
+	    }
+	    rpmfsSetAction(fs, i, FA_TOUCH);
+	    break;
 	default:
 	    break;
 	}
@@ -1037,8 +1049,6 @@ rpmdbMatchIterator rpmFindBaseNamesInDB(rpmts ts, uint64_t fileCount)
 
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, 0)) != NULL) {
-	(void) rpmsqPoll();
-
 	rpmtsNotify(ts, NULL, RPMCALLBACK_TRANS_PROGRESS, oc++, tsmem->orderCount);
 
 	/* Gather all installed headers with matching basename's. */
@@ -1442,7 +1452,7 @@ static int runTransScripts(rpmts ts, pkgGoal goal)
 
 static int rpmtsSetup(rpmts ts, rpmprobFilterFlags ignoreSet)
 {
-    rpm_tid_t tid = (rpm_tid_t) time(NULL);
+    rpm_tid_t tid = (rpm_tid_t) rpmtsGetTime(ts, 0);
     int dbmode = (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST) ?  O_RDONLY : (O_RDWR|O_CREAT);
 
     if (rpmtsFlags(ts) & RPMTRANS_FLAG_NOSCRIPTS)
@@ -1765,7 +1775,10 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     int TsmPreDone = 0; /* TsmPre hook hasn't been called */
     int nelem = rpmtsNElements(ts);
     /* Ignore SIGPIPE for the duration of transaction */
-    rpmsqAction_t oact = rpmsqSetAction(SIGPIPE, RPMSQ_IGN);
+    struct sigaction act, oact;
+    memset(&act, 0, sizeof(act));
+    act.sa_handler = SIG_IGN;
+    sigaction(SIGPIPE, &act, &oact);
     
     /* Force default 022 umask during transaction for consistent results */
     mode_t oldmask = umask(022);
@@ -1878,6 +1891,6 @@ exit:
     rpmpsFree(tsprobs);
     rpmtxnEnd(txn);
     /* Restore SIGPIPE *after* unblocking signals in rpmtxnEnd() */
-    rpmsqSetAction(SIGPIPE, oact);
+    sigaction(SIGPIPE, &oact, NULL);
     return rc;
 }
