@@ -1,14 +1,14 @@
 /* rpmarchive: spit out the main archive portion of a package */
 
 #include "system.h"
-const char *__progname;
 
 #include <rpm/rpmlib.h>		/* rpmReadPackageFile .. */
+#include <rpm/rpmstring.h>
 #include <rpm/rpmtag.h>
 #include <rpm/rpmio.h>
-#include <rpm/rpmpgp.h>
 
 #include <rpm/rpmts.h>
+#include <unistd.h>
 
 #include "debug.h"
 
@@ -17,10 +17,12 @@ int main(int argc, char *argv[])
     FD_t fdi, fdo;
     Header h;
     char * rpmio_flags = NULL;
-    rpmRC rc;
+    int rc;
+    off_t payload_size;
     FD_t gzdi;
     
-    setprogname(argv[0]);	/* Retrofit glibc __progname */
+    xsetprogname(argv[0]); /* Portability call -- see system.h */
+
     rpmReadConfigFiles(NULL, NULL);
     if (argc == 1)
 	fdi = fdDup(STDIN_FILENO);
@@ -37,14 +39,18 @@ int main(int argc, char *argv[])
 		(argc == 1 ? "<stdin>" : argv[1]), Fstrerror(fdi));
 	exit(EXIT_FAILURE);
     }
+    if (isatty(STDOUT_FILENO)) {
+	fprintf(stderr, "Error: refusing to output cpio data to a terminal.\n");
+	exit(EXIT_FAILURE);
+    }
     fdo = fdDup(STDOUT_FILENO);
 
     {	rpmts ts = rpmtsCreate();
 	rpmVSFlags vsflags = 0;
 
 	/* XXX retain the ageless behavior of rpm2cpio */
-        vsflags |= _RPMVSF_NODIGESTS;
-        vsflags |= _RPMVSF_NOSIGNATURES;
+        vsflags |= RPMVSF_MASK_NODIGESTS;
+        vsflags |= RPMVSF_MASK_NOSIGNATURES;
         vsflags |= RPMVSF_NOHDRCHK;
 	(void) rpmtsSetVSFlags(ts, vsflags);
 
@@ -69,9 +75,15 @@ int main(int argc, char *argv[])
 	break;
     }
 
-    /* Retrieve type of payload compression. */
+    if (headerIsEntry(h, RPMTAG_LONGFILESIZES)) {
+	fprintf(stderr, _("files over 4GB not supported by cpio, use rpm2archive instead\n"));
+	exit(EXIT_FAILURE);
+    }
+
+    /* Retrieve payload size and compression type. */
     {	const char *compr = headerGetString(h, RPMTAG_PAYLOADCOMPRESSOR);
 	rpmio_flags = rstrscat(NULL, "r.", compr ? compr : "gzip", NULL);
+	payload_size = headerGetNumber(h, RPMTAG_LONGARCHIVESIZE);
     }
 
     gzdi = Fdopen(fdi, rpmio_flags);	/* XXX gzdi == fdi */
@@ -82,8 +94,8 @@ int main(int argc, char *argv[])
 	exit(EXIT_FAILURE);
     }
 
-    rc = ufdCopy(gzdi, fdo);
-    rc = (rc <= 0) ? EXIT_FAILURE : EXIT_SUCCESS;
+    rc = (ufdCopy(gzdi, fdo) == payload_size) ? EXIT_SUCCESS : EXIT_FAILURE;
+
     Fclose(fdo);
 
     Fclose(gzdi);	/* XXX gzdi == fdi */
