@@ -38,10 +38,8 @@ enum macroFlags_e {
     ME_FUNC	= (1 << 4),
 };
 
-#define ME_BUILTIN (ME_PARSE|ME_FUNC)
-
 typedef struct MacroBuf_s *MacroBuf;
-typedef size_t (*macroFunc)(MacroBuf mb, rpmMacroEntry me, ARGV_t argv);
+typedef void (*macroFunc)(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed);
 
 /*! The structure used to store a macro. */
 struct rpmMacroEntry_s {
@@ -197,6 +195,27 @@ findEntry(rpmMacroContext mc, const char *name, size_t namelen, size_t *pos)
     return NULL;
 }
 
+/**
+ * Create a new entry in the macro table.
+ * @param mc		macro context
+ * @param pos		insert position
+ * @return		address of slot in macro table
+ */
+static rpmMacroEntry *
+newEntry(rpmMacroContext mc, size_t pos)
+{
+    /* extend macro table */
+    const int delta = 256;
+    if (mc->n % delta == 0)
+	mc->tab = xrealloc(mc->tab, sizeof(rpmMacroEntry) * (mc->n + delta));
+    /* shift pos+ entries to the right */
+    memmove(mc->tab + pos + 1, mc->tab + pos, sizeof(rpmMacroEntry) * (mc->n - pos));
+    mc->n++;
+    /* make slot */
+    mc->tab[pos] = NULL;
+    return &mc->tab[pos];
+}
+
 /* =============================================================== */
 
 /**
@@ -507,7 +526,7 @@ static void mbAppendStr(MacroBuf mb, const char *str)
     mb->nb -= len;
 }
 
-static size_t doDnl(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doDnl(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     const char *se = argv[1];
     const char *start = se, *end;
@@ -515,7 +534,8 @@ static size_t doDnl(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
     while (*s && !iseol(*s))
 	s++;
     end = (*s != '\0') ? s + 1 : s;
-    return end - start;
+    if (parsed)
+	*parsed += end - start;
 }
 
 /**
@@ -601,14 +621,14 @@ validName(MacroBuf mb, const char *name, size_t namelen, const char *action)
     int rc = 0;
     int c;
 
-    /* Names must start with alphabetic or _ and be at least 3 chars */
-    if (!((c = *name) && (risalpha(c) || c == '_') && (namelen) > 2)) {
+    /* Names must start with alphabetic, or _ and be at least 2 chars */
+    if (!((c = *name) && (risalpha(c) || (c == '_' && namelen > 1)))) {
 	mbErr(mb, 1, _("Macro %%%s has illegal name (%s)\n"), name, action);
 	goto exit;
     }
 
     mep = findEntry(mb->mc, name, namelen, NULL);
-    if (mep && (*mep)->flags & ME_BUILTIN) {
+    if (mep && (*mep)->flags & (ME_FUNC|ME_AUTO)) {
 	mbErr(mb, 1, _("Macro %%%s is a built-in (%s)\n"), name, action);
 	goto exit;
     }
@@ -627,8 +647,8 @@ exit:
  * @param expandbody	should body be expanded?
  * @return		number of consumed characters
  */
-static size_t
-doDefine(MacroBuf mb, const char * se, int level, int expandbody)
+static void
+doDefine(MacroBuf mb, const char * se, int level, int expandbody, size_t *parsed)
 {
     const char *start = se;
     const char *s = se;
@@ -663,7 +683,11 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
     b = be = oe + 1;
     sbody = s;
     SKIPBLANK(s, c);
-    if (c == '{') {	/* XXX permit silent {...} grouping */
+    if (!parsed) {
+	strcpy(b, s);
+	be = b + strlen(b);
+	s += strlen(s);
+    } else if (c == '{') {	/* XXX permit silent {...} grouping */
 	if ((se = matchchar(s, c, '}')) == NULL) {
 	    mbErr(mb, 1, _("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
@@ -747,7 +771,8 @@ exit:
 	mb->error = 1;
     _free(buf);
     _free(ebody);
-    return se - start;
+    if (parsed)
+	*parsed += se - start;
 }
 
 /**
@@ -756,8 +781,8 @@ exit:
  * @param argv		macro arguments
  * @return		number of consumed characters
  */
-static size_t
-doUndefine(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void
+doUndefine(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     const char *n = argv[1];
     if (!validName(mb, n, strlen(n), "%undefine")) {
@@ -765,39 +790,34 @@ doUndefine(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
     } else {
 	popMacro(mb->mc, n);
     }
-
-    return 0;
 }
 
-static size_t doArgvDefine(MacroBuf mb, ARGV_t argv, int level, int expand)
+static void doArgvDefine(MacroBuf mb, ARGV_t argv, int level, int expand, size_t *parsed)
 {
     char *args = NULL;
-    size_t ret;
     const char *se = argv[1];
 
     /* handle the "programmatic" case where macro name is arg1 and body arg2 */
     if (argv[2])
 	se = args = rstrscat(NULL, argv[1], " ", argv[2], NULL);
 
-    ret = doDefine(mb, se, level, expand);
+    doDefine(mb, se, level, expand, parsed);
     free(args);
-    return ret;
 }
 
-static size_t doDef(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doDef(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    return doArgvDefine(mb, argv, mb->level, 0);
+    doArgvDefine(mb, argv, mb->level, 0, parsed);
 }
 
-static size_t doGlobal(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doGlobal(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    return doArgvDefine(mb, argv, RMIL_GLOBAL, 1);
+    doArgvDefine(mb, argv, RMIL_GLOBAL, 1, parsed);
 }
 
-static size_t doDump(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doDump(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     rpmDumpMacroTable(mb->mc, NULL);
-    return 0;
 }
 
 
@@ -927,7 +947,8 @@ freeArgs(MacroBuf mb)
     mb->me = NULL;
 }
 
-static void splitQuoted(ARGV_t *av, const char * str, const char * seps)
+RPM_GNUC_INTERNAL
+void splitQuoted(ARGV_t *av, const char * str, const char * seps)
 {
     const int qchar = 0x1f; /* ASCII unit separator */
     const char *s = str;
@@ -959,47 +980,78 @@ static void splitQuoted(ARGV_t *av, const char * str, const char * seps)
     }
 }
 
+RPM_GNUC_INTERNAL
+char *unsplitQuoted(ARGV_const_t av, const char *sep)
+{
+    const int qchar = 0x1f; /* ASCII unit separator */
+    size_t len = 0, seplen;
+    char *b, *buf;
+    ARGV_const_t av2;
+    if (!av || !*av)
+	return xstrdup("");
+    seplen = av[1] ? strlen(sep) : 0;
+    for (av2 = av; *av2; av2++)
+	len += strlen(*av2) + 2 + seplen;
+    b = buf = xmalloc(len + 1 - seplen);
+    for (av2 = av; *av2; av2++) {
+	*b++ = qchar;
+	strcpy(b, *av2);
+	b += strlen(b);
+	*b++ = qchar;
+	if (av2[1]) {
+	    strcpy(b, sep);
+	    b += seplen;
+	}
+    }
+    *b = 0;
+    return buf;
+}
+
 /**
  * Parse arguments (to next new line) for parameterized macro.
  * @param mb		macro expansion state
+ * @param me		macro entry slot
  * @param argvp		pointer to argv array to store the result
  * @param se		arguments to parse
  * @param lastc		stop parsing at lastc
+ * @param splitargs	split into words
  * @return		address to continue parsing
  */
 static const char *
-grabArgs(MacroBuf mb, ARGV_t *argvp, const char * se,
-		const char * lastc)
+grabArgs(MacroBuf mb, const rpmMacroEntry me, ARGV_t *argvp,
+		const char * se, const char * lastc, int splitargs)
 {
-    char *s = NULL;
-    /* Expand possible macros in arguments */
-    expandThis(mb, se, lastc-se, &s);
-    splitQuoted(argvp, s, " \t");
-    free(s);
+    if ((me->flags & ME_PARSE) != 0) {
+	/* Always unsplit and unexpanded */
+	argvAddN(argvp, se, lastc - se);
+    } else {
+	char *s = NULL;
+	/* Expand possible macros in arguments */
+	expandThis(mb, lastc == se ? "" : se, lastc - se, &s);
+	if (splitargs)
+	    splitQuoted(argvp, s, " \t");
+	else
+	    argvAdd(argvp, s);
+	free(s);
+    }
     return (*lastc == '\0') || (*lastc == '\n' && *(lastc-1) != '\\') ?
 	   lastc : lastc + 1;
 }
 
-static size_t doBody(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doBody(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     if (*argv[1]) {
-	char *buf = NULL;
-	if (expandThis(mb, argv[1], 0, &buf) == 0) {
-	    rpmMacroEntry *mep = findEntry(mb->mc, buf, 0, NULL);
-	    if (mep) {
-		mbAppendStr(mb, (*mep)->body);
-	    } else {
-		mbErr(mb, 1, _("no such macro: '%s'\n"), buf);
-	    }
-	    free(buf);
+	rpmMacroEntry *mep = findEntry(mb->mc, argv[1], 0, NULL);
+	if (mep) {
+	    mbAppendStr(mb, (*mep)->body);
+	} else {
+	    mbErr(mb, 1, _("no such macro: '%s'\n"), argv[1]);
 	}
     }
-    return 0;
 }
 
-static size_t doOutput(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv)
+static void doOutput(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    char *buf = NULL;
     int loglevel = RPMLOG_NOTICE; /* assume echo */
     if (rstreq("error", me->name)) {
 	loglevel = RPMLOG_ERR;
@@ -1007,14 +1059,10 @@ static size_t doOutput(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv)
     } else if (rstreq("warn", me->name)) {
 	loglevel = RPMLOG_WARNING;
     }
-
-    (void) expandThis(mb, argv[1], 0, &buf);
-    rpmlog(loglevel, "%s\n", buf);
-    _free(buf);
-    return 0;
+    rpmlog(loglevel, "%s\n", argv[1]);
 }
 
-static size_t doLua(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv)
+static void doLua(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     rpmlua lua = NULL; /* Global state. */
     const char *scriptbuf = argv[1];
@@ -1046,124 +1094,66 @@ static size_t doLua(MacroBuf mb,  rpmMacroEntry me, ARGV_t argv)
 	mbAppendStr(mb, printbuf);
 	free(printbuf);
     }
-    return 0;
 }
 
-static size_t
-doSP(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doSP(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    const char *b = "";
-    char *buf = NULL;
     char *s = NULL;
 
-    if (*argv[1]) {
-	expandThis(mb, argv[1], 0, &buf);
-	b = buf;
-    }
-
-    s = rstrscat(NULL, (*(me->name) == 'S') ? "%SOURCE" : "%PATCH", b, NULL);
+    s = rstrscat(NULL, (*(me->name) == 'S') ? "%SOURCE" : "%PATCH", argv[1], NULL);
     expandMacro(mb, s, 0);
     free(s);
-    free(buf);
-    return 0;
 }
 
-static size_t doUncompress(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
-{
-    rpmCompressedMagic compressed = COMPRESSED_OTHER;
-    char *b, *be, *buf = NULL;
-    size_t gn = strlen(argv[1]);
-    int c;
-
-    if (gn) {
-	expandThis(mb, argv[1], gn, &buf);
-	for (b = buf; (c = *b) && isblank(c);)
-	    b++;
-	for (be = b; (c = *be) && !isblank(c);)
-	    be++;
-	*be++ = '\0';
-    }
-
-    if (gn == 0 || *b == '\0')
-	goto exit;
-
-    if (rpmFileIsCompressed(b, &compressed))
-	mb->error = 1;
-
-    switch (compressed) {
-    default:
-    case COMPRESSED_NOT:
-	expandMacro(mb, "%__cat ", 0);
-	break;
-    case COMPRESSED_OTHER:
-	expandMacro(mb, "%__gzip -dc ", 0);
-	break;
-    case COMPRESSED_BZIP2:
-	expandMacro(mb, "%__bzip2 -dc ", 0);
-	break;
-    case COMPRESSED_ZIP:
-	expandMacro(mb, "%__unzip ", 0);
-	break;
-    case COMPRESSED_LZMA:
-    case COMPRESSED_XZ:
-	expandMacro(mb, "%__xz -dc ", 0);
-	break;
-    case COMPRESSED_LZIP:
-	expandMacro(mb, "%__lzip -dc ", 0);
-	break;
-    case COMPRESSED_LRZIP:
-	expandMacro(mb, "%__lrzip -dqo- ", 0);
-	break;
-    case COMPRESSED_7ZIP:
-	expandMacro(mb, "%__7zip x ", 0);
-	break;
-    case COMPRESSED_ZSTD:
-	expandMacro(mb, "%__zstd -dc ", 0);
-	break;
-    }
-    mbAppendStr(mb, buf);
-
-exit:
-    free(buf);
-    return 0;
-}
-
-static size_t doExpand(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doUncompress(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     if (*argv[1]) {
-	char *buf;
-	expandThis(mb, argv[1], 0, &buf);
-	expandMacro(mb, buf, 0);
-	free(buf);
+	expandMacro(mb, "%__rpmuncompress ", 0);
+	mbAppendStr(mb, argv[1]);
     }
-    return 0;
 }
 
-static size_t doVerbose(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doExpand(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    mbAppend(mb, rpmIsVerbose() ? '1' : '0');
-    return 0;
+    if (*argv[1])
+	expandMacro(mb, argv[1], 0);
 }
 
-static size_t
-doFoo(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doVerbose(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
+{
+    if (parsed || !argv || !argv[1]) {
+	mbAppend(mb, rpmIsVerbose() ? '1' : '0');
+    } else if (rpmIsVerbose() && *argv[1]) {
+	expandMacro(mb, argv[1], 0);
+    }
+}
+
+static void doShescape(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
+{
+    mbAppend(mb, '\'');
+    for (const char *s = argv[1]; *s != '\0'; s++) {
+	if (*s == '\'') {
+	    mbAppendStr(mb, "'\\''");
+	} else {
+	    mbAppend(mb, *s);
+	}
+    }
+    mbAppend(mb, '\'');
+}
+
+static void doFoo(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     char *buf = NULL;
     char *b = NULL;
-    int expand = (argv[1] != NULL && *argv[1]);
 
-    if (expand) {
-	(void) expandThis(mb, argv[1], 0, &buf);
-    } else {
-	buf = xmalloc(MACROBUFSIZ + strlen(me->name));
-	buf[0] = '\0';
-    }
     if (rstreq("basename", me->name)) {
+	buf = xstrdup(argv[1]);
 	if ((b = strrchr(buf, '/')) == NULL)
 	    b = buf;
 	else
 	    b++;
     } else if (rstreq("dirname", me->name)) {
+	buf = xstrdup(argv[1]);
 	if ((b = strrchr(buf, '/')) != NULL)
 	    *b = '\0';
 	b = buf;
@@ -1172,81 +1162,65 @@ doFoo(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
 	 * shrink body by removing all leading and trailing whitespaces and
 	 * reducing intermediate whitespaces to a single space character.
 	 */
-	size_t i = 0, j = 0;
-	size_t buflen = strlen(buf);
-	int was_space = 0;
-	while (i < buflen) {
-	    if (risspace(buf[i])) {
-		was_space = 1;
-		i++;
-		continue;
-	    } else if (was_space) {
-		was_space = 0;
-		if (j > 0) /* remove leading blanks at all */
-		    buf[j++] = ' ';
+	char *p, c, last_c = ' ';
+	buf = b = p = xstrdup(argv[1]);
+	while ((c = *p++) != 0) {
+	    if (risspace(c)) {
+		if (last_c == ' ')
+		    continue;
+		c = ' ';
 	    }
-	    buf[j++] = buf[i++];
+	    *b++ = last_c = c;
 	}
-	buf[j] = '\0';
+	if (b != buf && b[-1] == ' ')
+	    b--;
+	*b = 0;
 	b = buf;
     } else if (rstreq("quote", me->name)) {
-	char *quoted = NULL;
-	rasprintf(&quoted, "%c%s%c", 0x1f, buf, 0x1f);
-	free(buf);
-	b = buf = quoted;
+	b = buf = unsplitQuoted(argv + 1, " ");
     } else if (rstreq("suffix", me->name)) {
+	buf = xstrdup(argv[1]);
 	if ((b = strrchr(buf, '.')) != NULL)
 	    b++;
     } else if (rstreq("expr", me->name)) {
-	char *expr = rpmExprStrFlags(buf, 0);
-	if (expr) {
-	    free(buf);
-	    b = buf = expr;
-	} else {
+	b = buf = rpmExprStrFlags(argv[1], 0);
+	if (!b)
 	    mb->error = 1;
-	}
     } else if (rstreq("url2path", me->name) || rstreq("u2p", me->name)) {
+	buf = xstrdup(argv[1]);
 	(void)urlPath(buf, (const char **)&b);
 	if (*b == '\0') b = "/";
     } else if (rstreq("getenv", me->name)) {
-	b = getenv(buf);
+	b = getenv(argv[1]);
     } else if (rstreq("getconfdir", me->name)) {
-	sprintf(buf, "%s", rpmConfigDir());
-	b = buf;
+	b = (char *)rpmConfigDir();
     } else if (rstreq("getncpus", me->name)) {
+	b = buf = xmalloc(MACROBUFSIZ);
 	sprintf(buf, "%u", getncpus());
-	b = buf;
     } else if (rstreq("exists", me->name)) {
-	b = (access(buf, F_OK) == 0) ? "1" : "0";
+	b = (access(argv[1], F_OK) == 0) ? "1" : "0";
     }
 
     if (b) {
 	mbAppendStr(mb, b);
     }
     free(buf);
-    return 0;
 }
 
-static size_t doLoad(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doLoad(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
-    char *arg = NULL;
-    if (expandThis(mb, argv[1], 0, &arg) == 0) {
-	if (loadMacroFile(mb->mc, arg)) {
-	    mbErr(mb, 1, _("failed to load macro file %s\n"), arg);
-	}
+    if (loadMacroFile(mb->mc, argv[1])) {
+	mbErr(mb, 1, _("failed to load macro file %s\n"), argv[1]);
     }
-    free(arg);
-    return 0;
 }
 
-static size_t doTrace(MacroBuf mb, rpmMacroEntry me, ARGV_t argv)
+static void doTrace(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
 {
     mb->expand_trace = mb->macro_trace = mb->depth;
     if (mb->depth == 1) {
 	print_macro_trace = mb->macro_trace;
 	print_expand_trace = mb->expand_trace;
     }
-    return 0;
 }
 
 static struct builtins_s {
@@ -1255,35 +1229,36 @@ static struct builtins_s {
     int nargs;
     int flags;
 } const builtinmacros[] = {
-    { "P",		doSP,		1,	ME_FUNC },
-    { "S",		doSP,		1,	ME_FUNC },
-    { "basename",	doFoo,		1,	ME_FUNC },
-    { "define",		doDef,		-1,	ME_PARSE },
-    { "dirname",	doFoo,		1,	ME_FUNC },
-    { "dnl",		doDnl,		-1,	ME_PARSE },
-    { "dump", 		doDump,		0,	ME_FUNC },
-    { "echo",		doOutput,	1,	ME_FUNC },
-    { "error",		doOutput,	1,	ME_FUNC },
-    { "exists",		doFoo,		1,	ME_FUNC },
-    { "expand",		doExpand,	1,	ME_FUNC },
-    { "expr",		doFoo,		1,	ME_FUNC },
-    { "getconfdir",	doFoo,		0,	ME_FUNC },
-    { "getenv",		doFoo,		1,	ME_FUNC },
-    { "getncpus",	doFoo,		0,	ME_FUNC },
-    { "global",		doGlobal,	-1,	ME_PARSE },
-    { "load",		doLoad,		1,	ME_FUNC },
-    { "lua",		doLua,		1,	ME_FUNC },
-    { "macrobody",	doBody,		1,	ME_FUNC },
-    { "quote",		doFoo,		1,	ME_FUNC },
-    { "shrink",		doFoo,		1,	ME_FUNC },
-    { "suffix",		doFoo,		1,	ME_FUNC },
-    { "trace",		doTrace,	0,	ME_FUNC },
-    { "u2p",		doFoo,		1,	ME_FUNC },
-    { "uncompress",	doUncompress,	1,	ME_FUNC },
-    { "undefine",	doUndefine,	1,	ME_FUNC },
-    { "url2path",	doFoo,		1,	ME_FUNC },
-    { "verbose",	doVerbose,	0,	ME_FUNC },
-    { "warn",		doOutput,	1,	ME_FUNC },
+    { "P",		doSP,		1,	0 },
+    { "S",		doSP,		1,	0 },
+    { "basename",	doFoo,		1,	0 },
+    { "define",		doDef,		1,	ME_PARSE },
+    { "dirname",	doFoo,		1,	0 },
+    { "dnl",		doDnl,		1,	ME_PARSE },
+    { "dump", 		doDump,		0,	0 },
+    { "echo",		doOutput,	1,	0 },
+    { "error",		doOutput,	1,	0 },
+    { "exists",		doFoo,		1,	0 },
+    { "expand",		doExpand,	1,	0 },
+    { "expr",		doFoo,		1,	0 },
+    { "getconfdir",	doFoo,		0,	0 },
+    { "getenv",		doFoo,		1,	0 },
+    { "getncpus",	doFoo,		0,	0 },
+    { "global",		doGlobal,	1,	ME_PARSE },
+    { "load",		doLoad,		1,	0 },
+    { "lua",		doLua,		1,	ME_PARSE },
+    { "macrobody",	doBody,		1,	0 },
+    { "quote",		doFoo,		1,	0 },
+    { "shrink",		doFoo,		1,	0 },
+    { "suffix",		doFoo,		1,	0 },
+    { "trace",		doTrace,	0,	0 },
+    { "u2p",		doFoo,		1,	0 },
+    { "shescape",	doShescape,	1,	0 },
+    { "uncompress",	doUncompress,	1,	0 },
+    { "undefine",	doUndefine,	1,	0 },
+    { "url2path",	doFoo,		1,	0 },
+    { "verbose",	doVerbose,	-1,	ME_PARSE },
+    { "warn",		doOutput,	1,	0 },
     { NULL,		NULL,		0 }
 };
 
@@ -1337,33 +1312,30 @@ const char *findMacroEnd(const char *str)
  * @param parsed	how many characters ME_PARSE parsed (or NULL)
  */
 static void
-doExpandThisMacro(MacroBuf mb, rpmMacroEntry me, ARGV_t args, size_t *parsed)
+doMacro(MacroBuf mb, rpmMacroEntry me, ARGV_t args, size_t *parsed)
 {
     rpmMacroEntry prevme = mb->me;
     ARGV_t prevarg = mb->args;
 
     /* Recursively expand body of macro */
-    if (me->flags & ME_BUILTIN) {
-	int nargs = argvCount(args) - 1;
+    if (me->flags & ME_FUNC) {
+	int nargs = args && args[0] ? (argvCount(args) - 1) : 0;
 	int needarg = (me->nargs != 0);
 	int havearg = (nargs > 0);
-	if (needarg != havearg) {
+	if (me->nargs >= 0 && needarg != havearg) {
 	    mbErr(mb, 1, "%%%s: %s\n", me->name, needarg ?
 		    _("argument expected") : _("unexpected argument"));
 	    goto exit;
 	}
-	size_t np = me->func(mb, me, args);
-	if (parsed)
-	    *parsed += np;
+	me->func(mb, me, args, parsed);
+    } else if (me->flags & ME_LITERAL) {
+	if (me->body && *me->body)
+	    mbAppendStr(mb, me->body);
     } else if (me->body && *me->body) {
 	/* Setup args for "%name " macros with opts */
 	if (args != NULL)
 	    setupArgs(mb, me, args);
-
-	if ((me->flags & ME_LITERAL) != 0)
-	    mbAppendStr(mb, me->body);
-	else
-	    expandMacro(mb, me->body, 0);
+	expandMacro(mb, me->body, 0);
 	/* Free args for "%name " macros with opts */
 	if (args != NULL)
 	    freeArgs(mb);
@@ -1521,7 +1493,7 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    if (g && g < ge) {		/* Expand X in %{...:X} */
 		expandMacro(mb, g, gn);
 	    } else if (me) {
-		doExpandThisMacro(mb, me, NULL, NULL);
+		doMacro(mb, me, NULL, NULL);
 	    }
 	    s = se;
 	    continue;
@@ -1534,23 +1506,33 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    continue;
 	}
 
-	/* Grab args for parametric macros */
-	ARGV_t args = NULL;
-	size_t fwd = 0;
-	if (me->opts != NULL) {
-	    argvAdd(&args, me->name);
-	    if (g) {
-		argvAddN(&args, g, gn);
-	    } else if (me->flags & ME_PARSE) {
-		argvAdd(&args, se);
-	    } else if (lastc) {
-		fwd = (grabArgs(mb, &args, fe, lastc) - se);
-	    }
+	if (me->opts == NULL && !(me->flags & ME_FUNC)) {
+	    /* Simple non-parametric macro */
+	    doMacro(mb, me, NULL, NULL);
+	    s = se;
+	    continue;
 	}
-	doExpandThisMacro(mb, me, args, &fwd);
-	if (args != NULL)
-	    argvFree(args);
-	s = se + (g ? 0 : fwd);
+	if (me->opts == NULL && fe == se)
+	    lastc = NULL;	/* do not parse arguments coming after %foo */
+
+	ARGV_t args = NULL;
+	argvAdd(&args, me->name);
+	if ((me->flags & ME_PARSE) != 0 && fe == se) {
+	    /* Special free-field macros define/global/dnl */
+	    size_t fwd = 0;
+	    argvAdd(&args, se);
+	    doMacro(mb, me, args, &fwd);
+	    se += fwd;
+	} else {
+	    /* Grab args for parametric macros */
+	    if (g)
+		se = grabArgs(mb, me, &args, g, ge, 0);
+	    else if (lastc)
+		se = grabArgs(mb, me, &args, fe, lastc, 1);
+	    doMacro(mb, me, args, NULL);
+	}
+	argvFree(args);
+	s = se;
     }
 
     mbFini(mb, me, &med);
@@ -1600,7 +1582,7 @@ expandThisMacro(MacroBuf mb, rpmMacroEntry me, ARGV_const_t args, int flags)
 	}
     }
 
-    doExpandThisMacro(mb, me, optargs, NULL);
+    doMacro(mb, me, optargs, NULL);
     if (optargs)
 	argvFree(optargs);
 
@@ -1645,42 +1627,28 @@ static void pushMacroAny(rpmMacroContext mc,
     if (mep) {
 	/* entry with shared name */
 	me = xmalloc(mesize);
-	/* copy body */
-	me->body = p = me->arena;
-	if (blen)
-	    memcpy(p, b, blen + 1);
-	else
-	    *p = '\0';
-	p += blen + 1;
+	p = me->arena;
 	/* set name */
 	me->name = (*mep)->name;
     }
     else {
-	/* extend macro table */
-	const int delta = 256;
-	if (mc->n % delta == 0)
-	    mc->tab = xrealloc(mc->tab, sizeof(me) * (mc->n + delta));
-	/* shift pos+ entries to the right */
-	memmove(mc->tab + pos + 1, mc->tab + pos, sizeof(me) * (mc->n - pos));
-	mc->n++;
-	/* make slot */
-	mc->tab[pos] = NULL;
-	mep = &mc->tab[pos];
 	/* entry with new name */
+	mep = newEntry(mc, pos);
 	size_t nlen = strlen(n);
 	me = xmalloc(mesize + nlen + 1);
-	/* copy body */
-	me->body = p = me->arena;
-	if (blen)
-	    memcpy(p, b, blen + 1);
-	else
-	    *p = '\0';
-	p += blen + 1;
+	p = me->arena;
 	/* copy name */
-	me->name = memcpy(p, n, nlen + 1);
+	me->name = p;
+	memcpy(p, n, nlen + 1);
 	p += nlen + 1;
     }
-
+    /* copy body */
+    me->body = p;
+    if (blen)
+	memcpy(p, b, blen + 1);
+    else
+	*p = '\0';
+    p += blen + 1;
     /* copy options */
     if (olen)
 	me->opts = memcpy(p, o, olen + 1);
@@ -1731,10 +1699,11 @@ static int defineMacro(rpmMacroContext mc, const char * macro, int level)
 {
     MacroBuf mb = xcalloc(1, sizeof(*mb));
     int rc;
+    size_t parsed = 0;
 
     /* XXX just enough to get by */
     mb->mc = mc;
-    (void) doDefine(mb, macro, level, 0);
+    doDefine(mb, macro, level, 0, &parsed);
     rc = mb->error;
     _free(mb);
     return rc;
@@ -1955,8 +1924,8 @@ rpmInitMacros(rpmMacroContext mc, const char * macrofiles)
 
     /* Define built-in macros */
     for (const struct builtins_s *b = builtinmacros; b->name; b++) {
-	pushMacroAny(mc, b->name, "", "<builtin>", b->func, b->nargs,
-			RMIL_BUILTIN, b->flags);
+	pushMacroAny(mc, b->name, b->nargs ? "" : NULL, "<builtin>", b->func, b->nargs,
+			RMIL_BUILTIN, b->flags | ME_FUNC);
     }
 
     argvSplit(&globs, macrofiles, ":");
