@@ -4,7 +4,7 @@
 typedef union {
     const char * s;
     int i;
-    float f;
+    double f;
     void * p;
 } rpmhookArgv;
 
@@ -16,6 +16,10 @@ typedef struct rpmhookArgs_s {
 
 typedef int (*rpmhookFunc) (rpmhookArgs args, void *data);
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+ 
 rpmhookArgs rpmhookArgsNew(int argc);
 rpmhookArgs rpmhookArgsFree(rpmhookArgs args);
 
@@ -26,4 +30,8 @@ void rpmhookUnregisterAll(const char *name);
 void rpmhookCall(const char *name, const char *argt, ...);
 void rpmhookCallArgs(const char *name, rpmhookArgs args);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
